#include "Configuration.h"
#include "ConnectionSettings.h"
#include "StartupSettings.h"
#include "DisplaySettings.h"
#include "Room.h"
#include <QDomDocument>
#include <QDomElement>
#include <QDomNode>
#include <QFile>

#include <QtDebug>

Configuration * Configuration::sInstance = 0;
QString Configuration::sConfigurationPath = "/etc/QtMeetings.conf";

Configuration::Configuration() :
		iConnectionSettings( 0 ),
		iStartupSettings( 0 ),
		iDisplaySettings( 0 )
{
}

Configuration::~Configuration()
{
	delete iConnectionSettings;
	delete iStartupSettings;
	delete iDisplaySettings;
	while ( !iRooms.isEmpty() )
	{
		delete iRooms.takeFirst();
	}
}

Configuration* Configuration::instance()
{
	if ( sInstance == 0 )
	{
		sInstance = readFromXML( sConfigurationPath );
	}
	return sInstance;
}

ConnectionSettings* Configuration::connectionSettings()
{
	return iConnectionSettings;
}

StartupSettings * Configuration::startupSettings()
{
	return iStartupSettings;
}

DisplaySettings * Configuration::displaySettings()
{
	return iDisplaySettings;
}

Room* Configuration::defaultRoom()
{
	// default room is stored as the first element of the list
	return ( iRooms.count() == 0 ) ? 0 : iRooms.at( 0 );
}

QString Configuration::languageCode()
{
	return iLanguageCode;
}

QList<Room*> Configuration::rooms()
{
	return iRooms;
}

QString Configuration::adminPassword()
{
	return iAdminPassword;
}

Configuration* Configuration::readFromXML( const QString &aPath )
{
	QDomDocument doc;
	QFile file( aPath );

	if ( !file.open( QIODevice::ReadOnly ) )
	{
		return 0;
	}
	if ( !doc.setContent( &file ) )
	{
		file.close();
		return 0;
	}
	file.close();

	QDomElement root = doc.documentElement();
	// check if the file is the one we need
	if ( root.tagName().toLower() != QString( "configuration" ) )
	{
		return 0;
	}

	Configuration* conf = new Configuration();

	conf->iAdminPassword = Configuration::readAdminPassword( root );

	for ( QDomNode node = root.firstChild(); !node.isNull(); node = node.nextSibling() )
	{
		QDomElement e = node.toElement();
		QString tagName = e.tagName().toLower();

		if ( tagName == QString( "connection" ) )
		{
			conf->iConnectionSettings = Configuration::readConnectionSettings( node );
		}
		else if ( tagName == QString( "rooms" ) )
		{
			conf->iRooms = Configuration::readRooms( node );
		}
		else if ( tagName == QString( "language" ) )
		{
			conf->iLanguageCode = Configuration::readLanguageCode( node );
		}
		else if ( tagName == QString( "startup" ) )
		{
			conf->iStartupSettings = Configuration::readStartupSettings( node );
		}
		else if ( tagName == QString( "display" ) )
		{
			conf->iDisplaySettings = Configuration::readDisplaySettings( node );
		}
	}

	return conf;
}

ConnectionSettings* Configuration::readConnectionSettings( const QDomNode &aXML )
{
	QString server, username, password;
	unsigned int interval = Configuration::sDefaultInterval;

	for ( QDomNode node = aXML.firstChild(); !node.isNull(); node = node.nextSibling() )
	{
		QDomElement e = node.toElement();
		QString tagName = e.tagName().toLower();

		if ( tagName == QString( "serverurl" ) )
		{
			server = e.text();
		}
		else if ( tagName == QString( "username" ) )
		{
			username = e.text();
		}
		else if ( tagName == QString( "password" ) )
		{
			password = e.text();
		}
		else if ( tagName == QString( "refreshinterval" ) )
		{
			bool success = false;
			unsigned int intervalTMP = e.text().toUInt( &success );
			if ( success )
			{
				interval = intervalTMP;
			}
		}
	}

	return new ConnectionSettings( server, username, password, interval );
}

QList<Room*> Configuration::readRooms( const QDomNode &aXML )
{
	QList<Room*> rooms;

	for ( QDomNode node = aXML.firstChild(); !node.isNull(); node = node.nextSibling() )
	{
		QDomElement e = node.toElement();
		QString tagName = e.tagName().toLower();

		if ( tagName == QString( "room" ) )
		{
			QString name, address;

			for ( QDomNode roomNode = node.firstChild(); !roomNode.isNull(); roomNode = roomNode.nextSibling() )
			{
				QDomElement roomElem = roomNode.toElement();
				tagName = roomElem.tagName().toLower();
				if ( tagName == QString( "name" ) )
				{
					name = roomElem.text();
				}
				else if ( tagName == QString( "address" ) )
				{
					address = roomElem.text();
				}
			}
			Room* room = new Room( name, address );
			QString defaultAttr = e.attribute( "default" ).toLower();
			if ( defaultAttr == QString( "true" ) )
			{
				rooms.insert( 0, room );
			}
			else
			{
				rooms.append( room );
			}
		}
	}

	return rooms;
}

QString Configuration::readLanguageCode( const QDomNode &aXML )
{
	QDomElement e = aXML.toElement();
	QString tagName = e.tagName().toLower();

	if ( e.hasAttribute( "code" ) )
	{
		return e.attribute( "code" );
	}
	else
	{
		// default language is English
		return "EN";
	}
}

StartupSettings * Configuration::readStartupSettings( const QDomNode &aXML )
{
	QDomElement e = aXML.toElement();

	bool isPowersavingEnabled = false;
	QTime turnOnAt, turnOffAt;

	for ( QDomNode node = aXML.firstChild(); !node.isNull(); node = node.nextSibling() )
	{
		e = node.toElement();
		QString tagName = e.tagName().toLower();

		if ( tagName == QString( "powersaving" ) )
		{
			if ( e.hasAttribute( "enabled" ) &&
				  e.attribute( "enabled" ) == QString( "true" ) )
			{
				isPowersavingEnabled = true;
			}

			if ( e.hasAttribute( "on" ) )
			{
				QString on = e.attribute( "on" );
				turnOnAt = QTime::fromString( on, "hh:mm" );

			}

			if ( e.hasAttribute( "off" ) )
			{
				QString off = e.attribute( "off" );
				turnOffAt = QTime::fromString( off, "hh:mm" );
			}
		}
	}

	return new StartupSettings( isPowersavingEnabled, turnOnAt, turnOffAt );
}

DisplaySettings * Configuration::readDisplaySettings( const QDomNode &aXML )
{
	DisplaySettings::DaysInSchedule daysInSchedule;
	QTime dayStartsAt, dayEndsAt;
	DisplaySettings::DateFormat dateformat;
	DisplaySettings::TimeFormat timeformat;

	for ( QDomNode node = aXML.firstChild(); !node.isNull(); node = node.nextSibling() )
	{
		QDomElement e = node.toElement();
		QString tagName = e.tagName().toLower();

		if ( tagName == QString( "schedule" ) )
		{
			for ( QDomNode scheduleNode = node.firstChild(); !scheduleNode.isNull(); scheduleNode = scheduleNode.nextSibling() )
			{
				QDomElement scheduleElem = scheduleNode.toElement();
				tagName = scheduleElem.tagName().toLower();

				if ( tagName == QString( "week" ) )
				{
					if ( scheduleElem.hasAttribute( "days" ) )
					{
						// default value is 5, only other supported possibility is 7
						bool success = false;
						unsigned int days = scheduleElem.attribute( "days" ).toUInt( &success );
						daysInSchedule = ( success && days == 7 ) ? DisplaySettings::WholeWeekInSchedule : DisplaySettings::WeekdaysInSchedule;
					}

				}
				else if ( tagName == QString( "day" ) )
				{
					if ( scheduleElem.hasAttribute( "startsat" ) )
					{
						QString time = scheduleElem.attribute( "startsat" );
						dayStartsAt = QTime::fromString( time, "hh:mm" );
					}
					if ( scheduleElem.hasAttribute( "endsat" ) )
					{
						QString time = scheduleElem.attribute( "endsat" );
						dayEndsAt = QTime::fromString( time, "hh:mm" );
					}

				}
			}	// end of for
		}	// end of schedule
		else if ( tagName == QString( "dateformat" ) )
		{
			// TODO : modify reading to read correct date format
			dateformat = DisplaySettings::ShortDateFormat;
		}
		else if ( tagName == QString( "timeformat" ) )
		{
			// TODO : modify reading to read correct time format
			timeformat = DisplaySettings::TwentyFourHoursTimeFormat;
		}
	}

	return new DisplaySettings( dateformat, timeformat, daysInSchedule, dayStartsAt, dayEndsAt );
}

QString Configuration::readAdminPassword( const QDomNode &aXML )
{
	QDomElement e = aXML.toElement();
	QString tagName = e.tagName().toLower();
	if ( e.hasAttribute( "password" ) )
	{
		return e.attribute( "password" );
	}
	else
	{
		return 0;
	}
}

