#ifndef CONFIGURATION_H_
#define CONFIGURATION_H_

#include <QString>
#include <QList>
#include <QTime>

class ConnectionSettings;
class StartupSettings;
class DisplaySettings;
class Room;
class QDomNode;

//! Domain class to read application settings.
/*!
 * Configuration reads and stores values read from configuration file.
 */
class Configuration
{
private:
	//! Constructor.
	/*!
	 * Constructor to initialize an Configuration instance.
	 */
	Configuration();

public:
	//! Destructor
	virtual ~Configuration();

	//! Configuration instance
	/*!
	 * Returns the static instance of this class.
	 * It is used to read the configuration information.
	 * \return Instance of this class.
	 */
	static Configuration* instance();

	/*!
	 * Returns the connection options.
	 * \return Pointer to ConnectionSettings instance.
	 */
	ConnectionSettings* connectionSettings();
	/*!
	 * Returns the default meeting room.
	 * \return Pointer to the default room.
	 */
	Room* defaultRoom();
	/*!
	 * Returns the language code.
	 * \return Language code
	 */
	QString languageCode();
	/*!
	 * Returns the list of meeting rooms available.
	 * \return List of rooms.
	 */
	QList<Room*> rooms();
	/*!
	 * Returns the startup options.
	 * \return Pointer to StartupSettings instance.
	 */
	StartupSettings* startupSettings();
	/*!
	 * Returns the display options.
	 * \return Pointer to DisplaySettings instance.
	 */
	DisplaySettings* displaySettings();
	/*!
	 * Returns the administration password
	 * \return Administration password.
	 */
	QString adminPassword();

private:
	/*!
	 * Static function to create configuration object and load the
	 * configuration information from configuration file
	 * \param aPath path and name of configuration file
	 * \return Configuration object.
	 */
	static Configuration* readFromXML( const QString &aPath );
	/*!
	 * Static function to create a Connection object by reading
	 * parameters from xml node
	 * \param aXml QDomNode containing connection parameters
	 * \return Pointer to ConnectionSettings object.
	 */
	static ConnectionSettings* readConnectionSettings( const QDomNode &aXML );
	/*!
	 * Static function to create a list of meetingrooms by reading
	 * parameters from xml node
	 * \param aXml QDomNode containing meeting room parameters
	 * \return List of meetingrooms.
	 */
	static QList<Room*> readRooms( const QDomNode &aXML );
	/*!
	 * Static function to read language code from xml node
	 * \param aXml QDomNode containing language code
	 * \return Language code.
	 */
	static QString readLanguageCode( const QDomNode &aXML );
	/*!
	 * Static function to load and store startup settings from xml node.
	 * \param aXml QDomNode containing startup parameters
	 * \return Pointer to the read StartupSettings object.
	 */
	static StartupSettings* readStartupSettings( const QDomNode &aXML );
	/*!
	 * Static function to load and store display settings from xml node.
	 * \param aXml QDomNode containing display parameters
	 * \return Pointer to the read DisplaySettings object.
	 */
	static DisplaySettings* readDisplaySettings( const QDomNode &aXML );
	/*!
	 * Static function to read admin password from xml node
	 * \param aXml QDomNode containing admin password
	 * \return Admin password.
	 */
	static QString readAdminPassword( const QDomNode &aXML );

private:
	//! Path and name of configuration file
	static QString sConfigurationPath;
	//! The static instance which is in use to read the configuration.
	static Configuration *sInstance;
	//! Static constant to store the default interval used for connection refresh.
	static const unsigned int sDefaultInterval = 60;
	//! Pointer to the ConnectionSettings object
	ConnectionSettings *iConnectionSettings;
	//! Stores startup settings.
	StartupSettings *iStartupSettings;
	//! Stores display settings.
	DisplaySettings *iDisplaySettings;
	//! List of meeting rooms.
	QList<Room*> iRooms;
	//! Stores administrator password.
	QString iAdminPassword;
	//! Stores language code
	QString iLanguageCode;

};

#endif /*CONFIGURATION_H_*/
