#include "AlarmSender.h"

#include <QtDebug>
#include <QFile>

const QString ALARMS_FILE = "/var/local/added_alarms.txt";

AlarmSender::AlarmSender()
{

}

AlarmSender::~AlarmSender()
{
}

void AlarmSender::sendAlarms( QTime aTurnOnAt, QTime aTurnOffAt, int aDays )
{
	if ( aDays != 5 ) //if aDays is 5 we set the alarms only for working days
		aDays = 7;

	if ( !removeAlarms() )   //remove possible old alarms
	{
		alarmSendingFailed( "Removing old alarms failed. Not sending any new alarms." );
		qDebug() << "Removing old alarms failed. Not sending any new alarms.";
		return;
	}

	alarm_event_t eventOn;
	alarm_event_t eventOff;

	//zero the alarm events
	memset( &eventOn, 0, sizeof( alarm_event_t ) );
	memset( &eventOff, 0, sizeof( alarm_event_t ) );

	//set the common event fields for the alarm events
	eventOn.recurrence = 60 * 24 * 7; //minutes in a week
	eventOff.recurrence = 60 * 24 * 7; //minutes in a week
	eventOn.recurrence_count = -1;
	eventOff.recurrence_count = -1;
	eventOn.flags = ( ALARM_EVENT_BOOT | ALARM_EVENT_NO_DIALOG );
	eventOff.flags = ALARM_EVENT_NO_DIALOG;
	eventOff.exec_name = "/usr/local/ixonosmeetings/devstopper";

	//find the first possible points of time to set the first alarms
	// Adjust the time. NOTE: alarms cannot be set to past
	QDateTime turnOnAtBasis = findFirstTime( aDays, aTurnOnAt );
	QDateTime turnOffAtBasis = findFirstTime( aDays, aTurnOffAt );

	cookie_t cookie;
	QList<long> alarmsSent;

	while ( alarmsSent.size() < aDays * 2 )
	{
		eventOn.alarm_time = turnOnAtBasis.toTime_t();
		cookie = alarm_event_add( &eventOn );
		alarmsSent.append( handleCookie( cookie ) );
		if ( cookie != 0 )
		{
			qDebug() << "Alarm event successfully sent to " << turnOnAtBasis.toString( "hh:mm:ss dd.MM.yyyy" );
		}
		turnOnAtBasis = turnOnAtBasis.addDays( 1 ); //move on to next day
		if ( aDays == 5 ) //only for working days
			turnOnAtBasis = turnOnAtBasis.addDays( daysToNextWorkingDay( turnOnAtBasis ) );

		eventOff.alarm_time = turnOffAtBasis.toTime_t();
		cookie = alarm_event_add( &eventOff );
		alarmsSent.append( handleCookie( cookie ) );
		if ( cookie != 0 )
		{
			qDebug() << "Alarm event successfully sent to " << turnOffAtBasis.toString( "hh:mm:ss dd.MM.yyyy" );
		}
		turnOffAtBasis = turnOffAtBasis.addDays( 1 ); //move on to next day
		if ( aDays == 5 ) //only for working days
			turnOffAtBasis = turnOffAtBasis.addDays( daysToNextWorkingDay( turnOffAtBasis ) );
	}
	if ( !storeSentAlarms( alarmsSent ) )
	{
		alarmSendingFailed( "Storing information about sent alarms failed. Removing already sent alarms." );
		qDebug() << "Storing information about sent alarms failed. Removing already sent alarms.";
		removeAlarms( alarmsSent );
	}

}

bool AlarmSender::removeAlarms()
{
	qDebug() << "Removing alarms 1.";
	QFile file( ALARMS_FILE );
	if ( file.exists() )
	{
		if ( !file.open( QIODevice::ReadOnly | QIODevice::Text ) )
			return false;

		QTextStream in( &file );
		while ( !in.atEnd() )
		{
			QString line = in.readLine();
			alarm_event_del( line.toLong() );
			qDebug() << "Deleted alarm " << line;
		}
		file.close();
		file.remove();
	}
	return true;
}

void AlarmSender::removeAlarms( QList<long> aAlarmsSent )
{
	qDebug() << "Removing alarms 2.";
	for ( int i = 0; i < aAlarmsSent.size(); ++i )
	{
		if ( aAlarmsSent.at( i ) != 0 )
		{
			qDebug() << "Removing alarm " << aAlarmsSent.at( i );
			alarm_event_del( aAlarmsSent.at( i ) );
		}
	}
}

QDateTime AlarmSender::findFirstTime( int aDays, QTime aTime )
{
	QDateTime currentTime = QDateTime::currentDateTime();
	QDateTime dateTimeBasis = QDateTime::currentDateTime();
	dateTimeBasis = dateTimeBasis.addSecs( dateTimeBasis.time().secsTo( aTime ) );

	if ( currentTime > dateTimeBasis )   //have to add a day
	{
		dateTimeBasis = dateTimeBasis.addDays( 1 );
	}
	if ( aDays == 5 )   //only for working days
	{
		dateTimeBasis = dateTimeBasis.addDays( daysToNextWorkingDay( dateTimeBasis ) );
	}
	return dateTimeBasis;
}

int AlarmSender::daysToNextWorkingDay( QDateTime aDateTime )
{
	QString day = aDateTime.toString( "ddd" );
	if ( day == "Sat" )
		return 2;
	else if ( day == "Sun" )
		return 1;
	else
		return 0;
}

long AlarmSender::handleCookie( cookie_t cookie )
{
	qDebug() << "Handlecookie " << cookie;
	if ( cookie == 0 )
	{
		QString errortext = "Error setting alarm event. ";
		errortext.append( mapError( alarmd_get_error() ) );
		qDebug() << errortext;
		emit alarmSendingFailed( errortext );
	}
	return ( long )cookie;
}

QString AlarmSender::mapError( alarm_error_t aErrorcode )
{
	QString errortext;
	switch ( aErrorcode )
	{
		case ALARMD_SUCCESS:
			errortext = "No reason found."; //this should never happen
			break;
		case ALARMD_ERROR_DBUS:
			errortext = "An error with D-Bus occurred, probably coudn't get a D-Bus connection.";
			break;
		case ALARMD_ERROR_CONNECTION:
			errortext = "Could not contact alarmd via D-Bus.";
			break;
		case ALARMD_ERROR_INTERNAL:
			errortext = "Some alarmd or libalarm internal error, possibly a version mismatch.";
			break;
		case ALARMD_ERROR_MEMORY:
			errortext = "A memory allocation failed.";
			break;
		case ALARMD_ERROR_ARGUMENT:
			errortext = "An argument given by caller was invalid.";
			break;
		default: /* == 6 */
			errortext = "Alarm daemon not running.";
	}
	return errortext;
}

bool AlarmSender::storeSentAlarms( QList<long> aAlarmsSent )
{
	QFile file( ALARMS_FILE );
	if ( !file.open( QIODevice::WriteOnly | QIODevice::Text ) )
		return false;

	QTextStream out( &file );
	for ( int i = 0; i < aAlarmsSent.size(); ++i )
	{
		if ( aAlarmsSent.at( i ) != 0 )
			out << aAlarmsSent.at( i ) << "\n";
	}
	file.close();
	return true;
}
