#ifndef SCHEDULEWIDGET_H_
#define SCHEDULEWIDGET_H_

#include "ObservedWidget.h"
#include <QDateTime>
#include <QList>
#include "DisplaySettings.h"
#include <QTableWidget>

class QTableWidgetItem;
class Meeting;

//! Custom QTableWidget class.
/*!
 * Subclassed QTableWidget for the purpose of drawing a main grid on top of
 * the standard one.
 */
class ScheduleTableWidget : public QTableWidget
{
	Q_OBJECT

	struct MeetingContainer
	{
		Meeting* meeting;
		QRect rect;
		bool rectComputed;
	};

public:

	//! Constructor.
	/*!
	 * \param aRows number of rows
	 * \param aColumns number of columns
	 * \param aParent parent widget
	 */
	ScheduleTableWidget( int aRows, int aColumns, QWidget *aParent = 0 );

	//! Destructor.
	/*!
	 *
	 */
	~ScheduleTableWidget();

protected:
	//! Paint event.
	/*!
	 * Handles drawing of the additional grid.
	 * \param aEvent event
	 */
	void paintEvent( QPaintEvent* aEvent );

	void tabletEvent( QTabletEvent* aEvent );

	void mouseMoveEvent( QMouseEvent* aEvent );

	void mousePressEvent( QMouseEvent* aEvent );

private:
	void PopulateMeetingList();

	bool FindOverlappingMeetings( int aDay, Meeting* aMeeting, QList<int>& aResult );

	void ActivateMeeting( int x, int y );

private:
	QList<MeetingContainer>* iMeetingsByDay;
};

//! ScheduleWidget class.
/*!
 * Custom widget for displaying a weekly schedule.
 */
class ScheduleWidget : public ObservedWidget
{
	Q_OBJECT

	friend class ScheduleTableWidget;

public:

	//! Constructor.
	/*!
	 * \param aCurrentDateTime Current date and time.
	 * \param aSettings Display settings.
	 * \param aParent Parent widget.
	 */
	ScheduleWidget( QDateTime aCurrentDateTime, DisplaySettings *aSettings, QWidget *aParent = 0 );
	//! Destructor.
	virtual ~ScheduleWidget();

	//! Current week.
	/*!
	 * \return current week.
	 */
	int	currentWeek();

	//! Shown week.
	/*!
	 * \return the week which is currently shown.
	 */
	int	shownWeek();

	//! Current meeting.
	/*!
	 * \return current meeting, 0 if none.
	 */
	Meeting* currentMeeting();

	//! Meeting.
	/*!
	 * \param aAt time when the meeting is.
	 * \return meeting at the given time, 0 if none.
	 */
	Meeting* meeting( QDateTime aAt );

signals:

	//! Meeting activated.
	/*!
	 * Signal is emitted when a meeting is clicked by the user.
	 * \param aMeeting actived meeting.
	 */
	void meetingActivated( Meeting *aMeeting );

	//! Meeting activated.
	/*!
	 * Signal is emitted when the current meeting changes to another.
	 * \param aNewMeeting new meeting.
	 */
	void currentMeetingChanged( Meeting *aNewMeeting );

	//! Shown week changed.
	/*!
	 * Signal is emitted when the week shown is changed.
	 * \param aWeekNumber week number.
	 */
	void shownWeekChanged( int aWeekNumber );

public slots:

	//! Clears all meetings.
	/*!
	 *
	 */
	void clear();

	//! Clears meetings based on a range.
	/*!
	 * \param aFrom start.
	 * \param aUntil end.
	 */
	void clear( QDateTime aFrom, QDateTime aUntil );

	//! Refreshes display.
	/*!
	 *
	 */
	void refresh();

	//! Sets current date and time.
	/*!
	 * \param aCurrentDateTime current date and time.
	 */
	void setCurrentDateTime( QDateTime aCurrentDateTime );

	//! Inserts a meeting.
	/*!
	 * \param aMeeting meeting.
	 */
	void insertMeeting( Meeting *aMeeting );

	//! Removes a meeting.
	/*!
	 * \param aMeeting meeting.
	 */
	void removeMeeting( Meeting *aMeeting );

	//! Updates a meeting (TODO).
	/*!
	 * \param aMeeting meeting.
	 */
	void updateMeeting( Meeting *aMeeting ) {};

	//! Shows previous week.
	/*!
	 *
	 */
	void showPreviousWeek();

	//! Shows current week.
	/*!
	 *
	 */
	void showCurrentWeek();

	//! Shows next week.
	/*!
	 *
	 */
	void showNextWeek();

private:

	//! Computes header row.
	/*!
	 * Computes header row number in schedule table based on given time.
	 * \param aTime time.
	 * \return header row.
	 */
	int computeHeaderRow( QTime aTime );

	//! Week length as days.
	/*!
	 * \return week length
	 */
	int weekLengthAsDays();

	//! Resize event.
	/*!
	 * Computes proper cell sizes for the schedule table.
	 * \param aEvent event.
	 */
	void resizeEvent( QResizeEvent *aEvent );

private:
	//! Schedule table widget.
	ScheduleTableWidget *iScheduleTable;

	//! Current date and time.
	QDateTime iCurrentDateTime;

	//! Meetings.
	QList<Meeting*> iMeetings;

	//! Currently shown week.
	QDate iShownDate;

	//! Starting hour of the schedule.
	int iStartHour;

	//! Number of hours in the schedule.
	int iNumberOfHours;

	//! Variable indicates the length of the week.
	DisplaySettings::DaysInSchedule iDaysInSchedule;

	//! When refresh() was called previously
	QTime iLastRefresh;

	//! Color for a free cell.
	static const QColor sFreeBackground;

	//! Color for a busy cell.
	static const QColor sBusyBackground;

	//! Color for headers.
	static const QColor sHeaderBackground;

	//! Color for time line.
	static const QColor sTimeLineBackground;

	//! Color for main grid.
	static const QColor sMainGridColor;

	//! Color for half grid.
	static const QColor sHalfGridColor;

	//! Color for frame.
	static const QColor sFrameColor;

	//! Refresh interval.
	static const int sRefreshIntervalInSeconds = 60;
};

#endif /*SCHEDULEWIDGET_H_*/
