#include "WindowManager.h"

#include <QApplication>
#include <QTimer>
#include "Configuration.h"
#include "DisplaySettings.h"
#include "Meeting.h"
#include "Room.h"
#include "Engine.h"
#include "Clock.h"
#include "WeeklyViewWidget.h"
#include "RoomStatusIndicatorWidget.h"
#include "MeetingInfoDialog.h"
#include "PopUpMessageBox.h"

#include <QtDebug>

const int IDLE_TIME = 60 * 1000; //Idle time is 1 minute

WindowManager::WindowManager() :
		QObject(),
		iApplicationName( tr( "Qt Meetings" ) ),
		iWeeklyView( 0 ),
		iRoomStatusView( 0 ),
		iMeetingInfo( 0 )
{
	iEngine = new Engine;
	connect( iEngine, SIGNAL( initializationFailed() ), this, SLOT( closeApplication() ) );
	connect( this, SIGNAL( roomStatusInfoNeeded( Room * ) ), iEngine, SLOT( roomStatusInfoNeeded( Room * ) ) );
	connect( iEngine, SIGNAL( roomStatusChanged( Room *, Room::Status, QTime ) ), this, SLOT( roomStatusChanged( Room *, Room::Status, QTime ) ) );
	connect( iEngine->clock(), SIGNAL( tick( QDateTime ) ), this, SLOT( distributeDateTimeInfo( QDateTime ) ) );
	connect( iEngine, SIGNAL( error( QString ) ), this, SLOT( error( QString ) ) );

	iWeeklyView = new WeeklyViewWidget( QDateTime::currentDateTime(), iEngine->configuration() );
	iWeeklyView->setWindowTitle( iApplicationName );
	connect( iEngine, SIGNAL( meetingAdded( Meeting * ) ), iWeeklyView, SLOT( insertMeeting( Meeting * ) ) );
	connect( iEngine, SIGNAL( meetingDeleted( Meeting * ) ), iWeeklyView, SLOT( deleteMeeting( Meeting * ) ) );
	connect( iWeeklyView, SIGNAL( observedEventDetected() ), this, SLOT( observedEventDetected() ) );
	connect( iWeeklyView, SIGNAL( meetingActivated( Meeting * ) ), this, SLOT( showMeetingInfo( Meeting * ) ) );

	iIdleTimeCounter = new QTimer();
	iIdleTimeCounter->setSingleShot( true );
	iIdleTimeCounter->setInterval( IDLE_TIME );
	iIdleTimeCounter->start();
	connect( iIdleTimeCounter, SIGNAL( timeout() ), this, SLOT( showRoomStatus() ) );

	showWeeklyView();

	//QTimer::singleShot( 0, this, SLOT( closeApplication() ) );
}

WindowManager::~WindowManager()
{
	if ( iWeeklyView != 0 )
	{
		delete iWeeklyView;
		iWeeklyView = 0;
	}

	if ( iRoomStatusView != 0 )
	{
		delete iRoomStatusView;
		iRoomStatusView = 0;
	}

	if ( iMeetingInfo != 0 )
	{
		delete iMeetingInfo;
		iMeetingInfo = 0;
	}

	if ( iIdleTimeCounter )
	{
		iIdleTimeCounter->stop();
		delete iIdleTimeCounter;
		iIdleTimeCounter = 0;
	}
}

void WindowManager::closeApplication()
{
	qDebug() << "WindowManager::closeApplication\tclose application";
	// closes application after 1 second
	QTimer::singleShot( 1000, QApplication::instance(), SLOT( quit() ) );
}

void WindowManager::distributeDateTimeInfo( QDateTime aCurrentDateTime )
{
	if ( iRoomStatusView != 0 && iRoomStatusView->isActiveWindow() )
	{
		iRoomStatusView->setCurrentTime( aCurrentDateTime.time() );
	}

	if ( iWeeklyView != 0 && iWeeklyView->isActiveWindow() )
	{
		iWeeklyView->setCurrentDateTime( aCurrentDateTime );
	}
}

void WindowManager::roomStatusChanged( Room *aRoom, Room::Status aStatus, QTime aTime )
{
	// currently works only for default room
	if ( aRoom->equals( iEngine->defaultRoom() ) )
	{
		if ( iRoomStatusView == 0 )
		{
			iRoomStatusView = new RoomStatusIndicatorWidget( aRoom, aStatus, aTime, iEngine->configuration()->displaySettings()->timeFormat() );
			iRoomStatusView->setWindowTitle( iApplicationName );
			connect( iRoomStatusView, SIGNAL( observedEventDetected() ), this, SLOT( observedEventDetected() ) );
		}
		else
		{
			iRoomStatusView->statusChanged( aStatus, aTime );
		}

		if ( !iWeeklyView->isVisible() && !iRoomStatusView->isVisible() )
		{
			iRoomStatusView->show();
		}
	}
}

void WindowManager::showRoomStatus()
{
	qDebug() << "WindowManager::showRoomStatus";

	if ( iRoomStatusView == 0 )
	{
		iEngine->roomStatusInfoNeeded( iWeeklyView->currentRoom() );
	}
	else
	{
		iRoomStatusView->show();
		if ( iWeeklyView->isVisible() )
		{
			iWeeklyView->hide();
		}
		if ( iMeetingInfo != 0 )
		{
			delete iMeetingInfo;
			iMeetingInfo = 0;
		}
	}
}

void WindowManager::showWeeklyView()
{
	qDebug() << "WindowManager::showWeeklyView";
	if ( iRoomStatusView != 0 && iRoomStatusView->isVisible() )
	{
		iRoomStatusView->hide();
	}

	iWeeklyView->show();
}

void WindowManager::showMeetingInfo( Meeting *aMeeting )
{
	iMeetingInfo = new MeetingInfoDialog( aMeeting );
	// Display modal dialog
	iMeetingInfo->exec();

	delete iMeetingInfo;
	iMeetingInfo = 0;
}

void WindowManager::error( const QString &aErrorMessage )
{
	qDebug() << "WindowManager::showErrorPopup";

	PopUpMessageBox::error( 0, aErrorMessage );
}

void WindowManager::observedEventDetected()
{
	// if event was detected on room status view
	if ( iRoomStatusView != 0 && iRoomStatusView->isVisible() )
	{
		// show weekly view
		showWeeklyView();
	}
	// otherwise
	else
	{
		// prepare to restart idle counter
		if ( iIdleTimeCounter->isActive() )
		{
			iIdleTimeCounter->stop();
		}
	}
	// (re)start idle counter
	iIdleTimeCounter->start();
}
